@@ -11,26 +11,27 @@
 #' @importFrom stats density nlminb
 #' @importFrom utils flush.console
 #'
-#' @param initialitem_D A matrix of initial dichotomous item parameter values for starting the estimation algorithm.
-#' @param initialitem_P A matrix of initial polytomous item parameter values for starting the estimation algorithm.
 #' @param data_D A matrix of item responses where responses are coded as 0 or 1.
 #' Rows and columns indicate examinees and items, respectively.
 #' @param data_P A matrix of polytomous item responses where responses are coded as \code{0, 1, ...}, or \code{m} for an \code{m+1} category item.
 #' Rows and columns indicate examinees and items, respectively.
-#' @param range Range of the latent variable to be considered in the quadrature scheme.
-#' The default is from \code{-6} to \code{6}: \code{c(-6, 6)}.
-#' @param q A numeric value that represents the number of quadrature points. The default value is 121.
 #' @param model_D A vector that represents types of item characteristic functions applied to each item.
 #' Insert \code{1}, \code{"1PL"}, \code{"Rasch"}, or \code{"RASCH"} for one-parameter logistic model,
 #' \code{2}, \code{"2PL"} for two-parameter logistic model,
-#' and \code{3}, \code{"3PL"} for three-parameter logistic model.
+#' and \code{3}, \code{"3PL"} for three-parameter logistic model. The default is \code{"2PL"}.
 #' @param model_P Currently, only the default (\code{"GPCM"}) is available.
+#' @param range Range of the latent variable to be considered in the quadrature scheme.
+#' The default is from \code{-6} to \code{6}: \code{c(-6, 6)}.
+#' @param q A numeric value that represents the number of quadrature points. The default value is 121.
+#' @param initialitem_D A matrix of initial dichotomous item parameter values for starting the estimation algorithm.
+#' @param initialitem_P A matrix of initial polytomous item parameter values for starting the estimation algorithm.
 #' @param latent_dist A character string that determines latent distribution estimation method.
 #' Insert \code{"Normal"}, \code{"normal"}, or \code{"N"} to assume normal distribution on the latent distribution,
 #' \code{"EHM"} for empirical histogram method (Mislevy, 1984; Mislevy & Bock, 1985),
 #' \code{"Mixture"} or \code{"2NM"} for the method of two-component Gaussian mixture distribution (Li, 2021; Mislevy, 1984),
 #' \code{"DC"} or \code{"Davidian"} for Davidian-curve method (Woods & Lin, 2009),
-#' and \code{"KDE"} for kernel density estimation method (Li, 2022).
+#' \code{"KDE"} for kernel density estimation method (Li, 2022),
+#' and \code{"LLS"} for log-linear smoothing method (Casabianca, Lewis, 2015).
 #' The default value is set to \code{"Normal"} for the conventional normality assumption on latent distribution.
 #'
 #' @param max_iter A numeric value that determines the maximum number of iterations in the EM-MML.
@@ -60,9 +61,15 @@
 #'
 #' }
 #' \item{
-#' Polytomous: The probability for scoring \eqn{k} (i.e., \eqn{u=k; k=0, 1, ..., m; m \ge 2}) in generalized partial credit model (GPCM) can be expressed as follows;
+#' Polytomous: The probability for scoring \eqn{k} (i.e., \eqn{u=k; k=0, 1, ..., m; m \ge 2})can be expressed as follows;
 #' }{
-#' 1) generalized partial credit model (GPCM)
+#' 1) partial credit model (PCM)
+#' \deqn{P(u=0|\theta, b_1, ..., b_{m})=\frac{1}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{a(\theta-b_v)}\right]}}}}
+#' \deqn{P(u=1|\theta, b_1, ..., b_{m})=\frac{\exp{(\theta-b_1)}}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{\theta-b_v}\right]}}}}
+#' \deqn{\vdots}
+#' \deqn{P(u=m|\theta, b_1, ..., b_{m})=\frac{\exp{\left[\sum_{v=1}^{m}{\theta-b_v}\right]}}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{\theta-b_v}\right]}}}}
+#'
+#' 2) generalized partial credit model (GPCM)
 #' \deqn{P(u=0|\theta, a, b_1, ..., b_{m})=\frac{1}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{a(\theta-b_v)}\right]}}}}
 #' \deqn{P(u=1|\theta, a, b_1, ..., b_{m})=\frac{\exp{(a(\theta-b_1))}}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{a(\theta-b_v)}\right]}}}}
 #' \deqn{\vdots}
@@ -92,6 +99,9 @@
 #' where \eqn{N} is the number of examinees, \eqn{\theta_j} is \eqn{j}th examinee's ability parameter, \eqn{h} is the bandwidth which corresponds to the argument \code{bw}, and \eqn{K( \bullet )} is a kernel function.
 #' The Gaussian kernel is used in this function.
 #'
+#' 5) Log-linear smoothing method
+#' \deqn{P(\theta=X_{q})=\exp{\beta_{0}+\sum_{m=1}^{h}{\beta_{m}X_{q}^{m}}}}
+#' where \eqn{h} is the hyper parameter which determines the smoothness of the density, and \eqn{\theta} can take total \eqn{Q} finite values (\eqn{X_1, \dots ,X_q, \dots, X_Q}).
 #' }
 #' }
 #'
@@ -104,13 +114,6 @@
 #' and The second object is the matrix of standard errors for polytomous items.}
 #' \item{fk}{The estimated frequencies of examinees at each quadrature points.}
 #' \item{iter}{The number of EM-MML iterations required for the convergence.}
-#' \item{prob}{The estimated \eqn{\pi = \frac{n_1}{N}} parameter of two-component Gaussian mixture distribution, where \eqn{n_1} is the estimated number of examinees who belong to the first Gaussian component and \eqn{N} is the total number of examinees (Li, 2021).}
-#' \item{d}{The estimated \eqn{\delta = \frac{\mu_2 - \mu_1}{\bar{\sigma}}} parameter of two-component Gaussian mixture distribution,
-#' where \eqn{\mu_1} is the estimated mean of the first Gaussian component,
-#' \eqn{\mu_2} is the estimated mean of the second Gaussian component,
-#' and \eqn{\bar{\sigma} = 1} is the standard deviation of the latent distribution (Li, 2021).
-#' Without loss of generality, \eqn{\mu_2 \ge \mu_1}, thus \eqn{\delta \ge 0}, is assumed.}
-#' \item{sd_ratio}{The estimated \eqn{\zeta = \frac{\sigma_2}{\sigma_1}} parameter of two-component Gaussian mixture distribution, where \eqn{\sigma_1} is the estimated standard deviation of the first Gaussian component, \eqn{\sigma_2} is the estimated standard deviation of the second Gaussian component (Li, 2021).}
 #' \item{quad}{The location of quadrature points.}
 #' \item{diff}{The final value of the monitored maximum item parameter change.}
 #' \item{Ak}{The estimated discrete latent distribution.
@@ -119,14 +122,25 @@
 #' \item{theta}{The estimated ability parameter values.
 #' Expected \emph{a posteriori} (EAP) is used for ability parameter estimation.}
 #' \item{logL}{The deviance (i.e., -2\emph{log}L).}
-#' \item{bw}{The bandwidth used.}
-#' \item{Options}{A replication of input arguments.}
+#' \item{density_par}{
+#' The estimated density parameters.
+#' If \code{latent_dist = "2NM"}, \code{prob} is the estimated \eqn{\pi = \frac{n_1}{N}} parameter of two-component Gaussian mixture distribution, where \eqn{n_1} is the estimated number of examinees who belong to the first Gaussian component and \eqn{N} is the total number of examinees;
+#' \code{d} is the estimated \eqn{\delta = \frac{\mu_2 - \mu_1}{\bar{\sigma}}} parameter of two-component Gaussian mixture distribution,
+#' where \eqn{\mu_1} is the estimated mean of the first Gaussian component,
+#' \eqn{\mu_2} is the estimated mean of the second Gaussian component,
+#' and \eqn{\bar{\sigma} = 1} is the standard deviation of the latent distribution; and
+#' \code{sd_ratio} is the estimated \eqn{\zeta = \frac{\sigma_2}{\sigma_1}} parameter of two-component Gaussian mixture distribution, where \eqn{\sigma_1} is the estimated standard deviation of the first Gaussian component, \eqn{\sigma_2} is the estimated standard deviation of the second Gaussian component (Li, 2021).
+#' Without loss of generality, \eqn{\mu_2 \ge \mu_1}, thus \eqn{\delta \ge 0}, is assumed.}
+#' \item{Options}{A replication of input arguments and other information.}
+#'
 #'
 #' @author Seewoo Li \email{cu@@yonsei.ac.kr}
 #'
 #' @references
 #' Bock, R. D., & Aitkin, M. (1981). Marginal maximum likelihood estimation of item parameters: Application of an EM algorithm. \emph{Psychometrika, 46}(4), 443-459.
 #'
+#' Casabianca, J. M., & Lewis, C. (2015). IRT item parameter recovery with marginal maximum likelihood estimation using loglinear smoothing models. \emph{Journal of Educational and Behavioral Statistics, 40}(6), 547-578.
+#'
 #' Lee, W. C., Kim, S. Y., Choi, J., & Kang, Y. (2020). IRT Approaches to Modeling Scores on Mixed-Format Tests. \emph{Journal of Educational Measurement, 57}(2), 230-254.
 #'
 #' Li, S. (2021). Using a two-component normal mixture distribution as a latent distribution in estimating parameters of item response models. \emph{Journal of Educational Evaluation, 34}(4), 759-789.
@@ -183,16 +197,37 @@
 #'                  threshold = .001,
 #'                  h=9)
 #'
-
-IRTest_Mix <- function(initialitem_D, initialitem_P, data_D, data_P, range = c(-6,6),
-                       q = 121, model_D, model_P="GPCM",
+IRTest_Mix <- function(data_D, data_P, model_D="2PL",
+                       model_P="GPCM", range = c(-6,6),q = 121,
+                       initialitem_D=NULL, initialitem_P=NULL,
                        latent_dist="Normal", max_iter=200, threshold=0.0001,
                        bandwidth="SJ-ste", h=NULL){
+
+  if(is.null(initialitem_D)){
+    initialitem_D <- matrix(rep(c(1,0,0), each = ncol(data_D)), ncol = 3)
+  }
+  if(length(model_D)==1){
+    model_D <- rep(model_D, ncol(data_D))
+  }
+
+  categories <- apply(data_P, MARGIN = 2, FUN = extract_cat)
+
+  data_P <- reorder_mat(as.matrix(data_P))
+  if(is.null(initialitem_P)){
+    category <- apply(data_P, 2, max, na.rm = TRUE)
+    initialitem_P <- matrix(nrow = ncol(data_P), ncol = 7)
+    initialitem_P[,1] <- 1
+    initialitem_P[,2] <- 0
+    for(i in 3:7){
+      initialitem_P[category>(i-2),i] <- 0
+    }
+  }
+
   Options = list(initialitem_D=initialitem_D, initialitem_P=initialitem_P,
                  data_D=data_D, data_P=data_P, range=range, q=q,
                  model_D=model_D, model_P=model_P,
                  latent_dist=latent_dist, max_iter=max_iter, threshold=threshold,
-                 bandwidth=bandwidth,h=h)
+                 bandwidth=bandwidth,h=h,categories=categories)
 if(nrow(data_D)!=nrow(data_P)){
   stop("data_D and data_P have different number of rows.")
 }else{
@@ -206,7 +241,7 @@ if(nrow(data_D)!=nrow(data_P)){
   d = 1
   sd_ratio = 1
   N = nrow(data_D)
-  bw <- NULL
+  density_par <- NULL
 
   # Normality assumption method
   if(latent_dist %in% c("Normal", "normal", "N")){
@@ -220,6 +255,17 @@ if(nrow(data_D)!=nrow(data_P)){
       initialitem_D <- M1_D[[1]]
       initialitem_P <- M1_P[[1]]
 
+      if(all(c(model_D, model_P) %in% c(1, "1PL", "Rasch", "RASCH", "PCM"))){
+        ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range)
+        initialitem_D[,1] <- initialitem_D[,1]*ld_est$s
+        initialitem_D[,2] <- initialitem_D[,2]/ld_est$s
+        M1_D[[2]][,2] <- M1_D[[2]][,2]/ld_est$s
+
+        initialitem_P[,1]  <- initialitem_P[,1]*ld_est$s
+        initialitem_P[,-1] <- initialitem_P[,-1]/ld_est$s
+        M1_P[[2]][,-1] <- M1_P[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(c(max(abs(I_D-initialitem_D), na.rm = TRUE), max(abs(I_P-initialitem_P), na.rm = TRUE)))
       I_D <- initialitem_D
       I_P <- initialitem_P
@@ -246,6 +292,16 @@ if(nrow(data_D)!=nrow(data_P)){
       Xk <- ld_est$Xk
       Ak <- ld_est$posterior_density
 
+      if(all(c(model_D, model_P) %in% c(1, "1PL", "Rasch", "RASCH", "PCM"))){
+        initialitem_D[,1] <- initialitem_D[,1]*ld_est$s
+        initialitem_D[,2] <- initialitem_D[,2]/ld_est$s
+        M1_D[[2]][,2] <- M1_D[[2]][,2]/ld_est$s
+
+        initialitem_P[,1]  <- initialitem_P[,1]*ld_est$s
+        initialitem_P[,-1] <- initialitem_P[,-1]/ld_est$s
+        M1_P[[2]][,-1] <- M1_P[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(c(max(abs(I_D-initialitem_D), na.rm = TRUE), max(abs(I_P-initialitem_P), na.rm = TRUE)))
       I_D <- initialitem_D
       I_P <- initialitem_P
@@ -268,7 +324,18 @@ if(nrow(data_D)!=nrow(data_P)){
       initialitem_P <- M1_P[[1]]
 
       M2 <- M2step(E)
-      prob = M2[1];d = M2[3];sd_ratio = M2[4]
+
+      if(all(c(model_D, model_P) %in% c(1, "1PL", "Rasch", "RASCH", "PCM"))){
+        initialitem_D[,1] <- initialitem_D[,1]*M2$s
+        initialitem_D[,2] <- initialitem_D[,2]/M2$s
+        M1_D[[2]][,2] <- M1_D[[2]][,2]/M2$s
+
+        initialitem_P[,1]  <- initialitem_P[,1]*M2$s
+        initialitem_P[,-1] <- initialitem_P[,-1]/M2$s
+        M1_P[[2]][,-1] <- M1_P[[2]][,-1]/M2$s
+      }
+
+      prob = M2$prob; d = M2$d; sd_ratio = M2$sd_ratio
       diff <- max(c(max(abs(I_D-initialitem_D), na.rm = TRUE), max(abs(I_P-initialitem_P), na.rm = TRUE)))
       I_D <- initialitem_D
       I_P <- initialitem_P
@@ -276,6 +343,7 @@ if(nrow(data_D)!=nrow(data_P)){
       flush.console()
     }
     Ak <- E$Ak
+    density_par <- list(prob=prob, d=d, sd_ratio=sd_ratio)
   }
 
   # Kernel density estimation method
@@ -295,6 +363,16 @@ if(nrow(data_D)!=nrow(data_P)){
       Xk <- ld_est$Xk
       Ak <- ld_est$posterior_density
 
+      if(all(c(model_D, model_P) %in% c(1, "1PL", "Rasch", "RASCH", "PCM"))){
+        initialitem_D[,1] <- initialitem_D[,1]*ld_est$s
+        initialitem_D[,2] <- initialitem_D[,2]/ld_est$s
+        M1_D[[2]][,2] <- M1_D[[2]][,2]/ld_est$s
+
+        initialitem_P[,1]  <- initialitem_P[,1]*ld_est$s
+        initialitem_P[,-1] <- initialitem_P[,-1]/ld_est$s
+        M1_P[[2]][,-1] <- M1_P[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(c(max(abs(I_D-initialitem_D), na.rm = TRUE), max(abs(I_P-initialitem_P), na.rm = TRUE)))
       I_D <- initialitem_D
       I_P <- initialitem_P
@@ -302,12 +380,12 @@ if(nrow(data_D)!=nrow(data_P)){
       message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
       flush.console()
     }
-    bw <- ld_est$bw
+    density_par <- ld_est$par
   }
 
   # Davidian curve method
   if(latent_dist %in% c("DC", "Davidian")){
-    phipar <- nlminb(start = rep(1,h),
+    density_par <- nlminb(start = rep(1,h),
                      objective = optim_phi,
                      gradient = optim_phi_grad,
                      hp=h,
@@ -325,10 +403,20 @@ if(nrow(data_D)!=nrow(data_P)){
       initialitem_D <- M1_D[[1]]
       initialitem_P <- M1_P[[1]]
 
-      ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range, phipar=phipar)
+      ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range, par=density_par)
       Xk <- ld_est$Xk
       Ak <- ld_est$posterior_density
 
+      if(all(c(model_D, model_P) %in% c(1, "1PL", "Rasch", "RASCH", "PCM"))){
+        initialitem_D[,1] <- initialitem_D[,1]*ld_est$s
+        initialitem_D[,2] <- initialitem_D[,2]/ld_est$s
+        M1_D[[2]][,2] <- M1_D[[2]][,2]/ld_est$s
+
+        initialitem_P[,1]  <- initialitem_P[,1]*ld_est$s
+        initialitem_P[,-1] <- initialitem_P[,-1]/ld_est$s
+        M1_P[[2]][,-1] <- M1_P[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(c(max(abs(I_D-initialitem_D), na.rm = TRUE), max(abs(I_P-initialitem_P), na.rm = TRUE)))
       I_D <- initialitem_D
       I_P <- initialitem_P
@@ -336,8 +424,48 @@ if(nrow(data_D)!=nrow(data_P)){
       message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
       flush.console()
     }
+    density_par <- ld_est$par
+  }
+
+  # Log-linear smoothing
+  if(latent_dist=="LLS"){
+    density_par <- rep(0, h)
+    while(iter < max_iter & diff > threshold){
+      iter <- iter +1
+
+      E <- Estep_Mix(item_D=initialitem_D, item_P=initialitem_P, data_D=data_D,
+                     data_P=data_P, q=q, prob=0.5, d=0, sd_ratio=1, range=range,
+                     Xk = Xk, Ak=Ak)
+      M1_D <- M1step(E, item=initialitem_D, model=model_D)
+      M1_P <- Mstep_Poly(E, item=initialitem_P, model=model_P)
+      initialitem_D <- M1_D[[1]]
+      initialitem_P <- M1_P[[1]]
+
+      ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range, par=density_par, N=N)
+      Xk <- ld_est$Xk
+      Ak <- ld_est$posterior_density
+
+      if(all(c(model_D, model_P) %in% c(1, "1PL", "Rasch", "RASCH", "PCM"))){
+        initialitem_D[,1] <- initialitem_D[,1]*ld_est$s
+        initialitem_D[,2] <- initialitem_D[,2]/ld_est$s
+        M1_D[[2]][,2] <- M1_D[[2]][,2]/ld_est$s
+
+        initialitem_P[,1]  <- initialitem_P[,1]*ld_est$s
+        initialitem_P[,-1] <- initialitem_P[,-1]/ld_est$s
+        M1_P[[2]][,-1] <- M1_P[[2]][,-1]/ld_est$s
+      }
+
+      diff <- max(c(max(abs(I_D-initialitem_D), na.rm = TRUE), max(abs(I_P-initialitem_P), na.rm = TRUE)))
+      I_D <- initialitem_D
+      I_P <- initialitem_P
+
+      message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
+      flush.console()
+    }
+    density_par <- ld_est$par
   }
 }
+
   colnames(initialitem_D) <- c("a", "b", "c")
   colnames(initialitem_P) <- c("a", "b_1", "b_2", "b_3", "b_4", "b_5", "b_6")
   colnames(M1_D[[2]]) <- c("a", "b", "c")
@@ -360,16 +488,13 @@ if(nrow(data_D)!=nrow(data_P)){
                  Polytomous=M1_P[[2]]),
          fk=E$fk,
          iter=iter,
-         prob=prob,
-         d=d,
-         sd_ratio=sd_ratio,
          quad=Xk,
          diff=diff,
          Ak=Ak,
          Pk=E$Pk,
          theta = EAP,
          logL=-2*logL, # deviance
-         bw=bw,
+         density_par = density_par,
          Options = Options # specified argument values
          ),
     class = c("mix", "irtest", "list")
