@@ -9,23 +9,26 @@
 #' @importFrom stats density nlminb
 #' @importFrom utils flush.console
 #'
-#' @param initialitem A matrix of initial item parameter values for starting the estimation algorithm.
-#' This matrix determines the number of categories for each item.
 #' @param data A matrix of item responses where responses are coded as \code{0, 1, ..., m} for an \code{m+1} category item.
 #' Rows and columns indicate examinees and items, respectively.
+#' @param model A character value that represents the type of a item characteristic function applied to the items.
+#' Currently, \code{PCM} and \code{GPCM} are available. The default is \code{"GPCM"}.
 #' @param range Range of the latent variable to be considered in the quadrature scheme.
 #' The default is from \code{-6} to \code{6}: \code{c(-6, 6)}.
 #' @param q A numeric value that represents the number of quadrature points. The default value is 121.
-#' @param model A vector that represents types of item characteristic functions applied to each item.
-#' However, only generalized partial credit model (\code{GPCM}) is currently available.
+#' @param initialitem A matrix of initial item parameter values for starting the estimation algorithm.
+#' This matrix determines the number of categories for each item.
+#' @param ability_method The ability parameter estimation method.
+#' The available options are Expected \emph{a posteriori} (\code{EAP}) and Maximum Likelihood Estimates (\code{MLE}).
+#' The default is \code{EAP}.
 #' @param latent_dist A character string that determines latent distribution estimation method.
 #' Insert \code{"Normal"}, \code{"normal"}, or \code{"N"} to assume normal distribution on the latent distribution,
 #' \code{"EHM"} for empirical histogram method (Mislevy, 1984; Mislevy & Bock, 1985),
 #' \code{"Mixture"} or \code{"2NM"} for the method of two-component Gaussian mixture distribution (Li, 2021; Mislevy, 1984),
 #' \code{"DC"} or \code{"Davidian"} for Davidian-curve method (Woods & Lin, 2009),
-#' and \code{"KDE"} for kernel density estimation method (Li, 2022).
+#' \code{"KDE"} for kernel density estimation method (Li, 2022),
+#' and \code{"LLS"} for log-linear smoothing method (Casabianca, Lewis, 2015).
 #' The default value is set to \code{"Normal"} for the conventional normality assumption on latent distribution.
-#'
 #' @param max_iter A numeric value that determines the maximum number of iterations in the EM-MML.
 #' The default value is 200.
 #' @param threshold A numeric value that determines the threshold of EM-MML convergence.
@@ -40,9 +43,15 @@
 #' @details
 #' \describe{
 #' \item{
-#' The probability for scoring \eqn{k} (i.e., \eqn{u=k; k=0, 1, ..., m; m \ge 2}) in generalized partial credit model (GPCM) can be expressed as follows;
+#' The probability for scoring \eqn{k} (i.e., \eqn{u=k; k=0, 1, ..., m; m \ge 2})can be expressed as follows;
 #' }{
-#' 1) generalized partial credit model (GPCM)
+#' 1) partial credit model (PCM)
+#' \deqn{P(u=0|\theta, b_1, ..., b_{m})=\frac{1}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{a(\theta-b_v)}\right]}}}}
+#' \deqn{P(u=1|\theta, b_1, ..., b_{m})=\frac{\exp{(\theta-b_1)}}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{\theta-b_v}\right]}}}}
+#' \deqn{\vdots}
+#' \deqn{P(u=m|\theta, b_1, ..., b_{m})=\frac{\exp{\left[\sum_{v=1}^{m}{\theta-b_v}\right]}}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{\theta-b_v}\right]}}}}
+#'
+#' 2) generalized partial credit model (GPCM)
 #' \deqn{P(u=0|\theta, a, b_1, ..., b_{m})=\frac{1}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{a(\theta-b_v)}\right]}}}}
 #' \deqn{P(u=1|\theta, a, b_1, ..., b_{m})=\frac{\exp{(a(\theta-b_1))}}{1+\sum_{c=1}^{m}{\exp{\left[\sum_{v=1}^{c}{a(\theta-b_v)}\right]}}}}
 #' \deqn{\vdots}
@@ -71,6 +80,10 @@
 #' where \eqn{N} is the number of examinees, \eqn{\theta_j} is \eqn{j}th examinee's ability parameter, \eqn{h} is the bandwidth which corresponds to the argument \code{bw}, and \eqn{K( \bullet )} is a kernel function.
 #' The Gaussian kernel is used in this function.
 #'
+#' 5) Log-linear smoothing method
+#' \deqn{P(\theta=X_{q})=\exp{\beta_{0}+\sum_{m=1}^{h}{\beta_{m}X_{q}^{m}}}}
+#' where \eqn{h} is the hyper parameter which determines the smoothness of the density, and \eqn{\theta} can take total \eqn{Q} finite values (\eqn{X_1, \dots ,X_q, \dots, X_Q}).
+#'
 #' }
 #' }
 #'
@@ -79,13 +92,6 @@
 #' \item{se}{The standard errors for item parameter estimates.}
 #' \item{fk}{The estimated frequencies of examinees at each quadrature points.}
 #' \item{iter}{The number of EM-MML iterations required for the convergence.}
-#' \item{prob}{The estimated \eqn{\pi = \frac{n_1}{N}} parameter of two-component Gaussian mixture distribution, where \eqn{n_1} is the estimated number of examinees who belong to the first Gaussian component and \eqn{N} is the total number of examinees (Li, 2021).}
-#' \item{d}{The estimated \eqn{\delta = \frac{\mu_2 - \mu_1}{\bar{\sigma}}} parameter of two-component Gaussian mixture distribution,
-#' where \eqn{\mu_1} is the estimated mean of the first Gaussian component,
-#' \eqn{\mu_2} is the estimated mean of the second Gaussian component,
-#' and \eqn{\bar{\sigma} = 1} is the standard deviation of the latent distribution (Li, 2021).
-#' Without loss of generality, \eqn{\mu_2 \ge \mu_1}, thus \eqn{\delta \ge 0}, is assumed.}
-#' \item{sd_ratio}{The estimated \eqn{\zeta = \frac{\sigma_2}{\sigma_1}} parameter of two-component Gaussian mixture distribution, where \eqn{\sigma_1} is the estimated standard deviation of the first Gaussian component, \eqn{\sigma_2} is the estimated standard deviation of the second Gaussian component (Li, 2021).}
 #' \item{quad}{The location of quadrature points.}
 #' \item{diff}{The final value of the monitored maximum item parameter change.}
 #' \item{Ak}{The estimated discrete latent distribution.
@@ -93,9 +99,20 @@
 #' \item{Pk}{The posterior probabilities for each examinees at each quadrature points.}
 #' \item{theta}{The estimated ability parameter values.
 #' Expected \emph{a posteriori} (EAP) is used for ability parameter estimation.}
+#' \item{theta_se}{The asymptotic standard errors of ability parameter estimates. Available only when \code{ability_method = "MLE"}.
+#' If an examinee answers all or none of the items correctly, the function returns \code{NA}.}
 #' \item{logL}{The deviance (i.e., -2\emph{log}L).}
-#' \item{bw}{The bandwidth used.}
-#' \item{Options}{A replication of input arguments.}
+#' \item{density_par}{
+#' The estimated density parameters.
+#' If \code{latent_dist = "2NM"}, \code{prob} is the estimated \eqn{\pi = \frac{n_1}{N}} parameter of two-component Gaussian mixture distribution, where \eqn{n_1} is the estimated number of examinees who belong to the first Gaussian component and \eqn{N} is the total number of examinees;
+#' \code{d} is the estimated \eqn{\delta = \frac{\mu_2 - \mu_1}{\bar{\sigma}}} parameter of two-component Gaussian mixture distribution,
+#' where \eqn{\mu_1} is the estimated mean of the first Gaussian component,
+#' \eqn{\mu_2} is the estimated mean of the second Gaussian component,
+#' and \eqn{\bar{\sigma} = 1} is the standard deviation of the latent distribution; and
+#' \code{sd_ratio} is the estimated \eqn{\zeta = \frac{\sigma_2}{\sigma_1}} parameter of two-component Gaussian mixture distribution, where \eqn{\sigma_1} is the estimated standard deviation of the first Gaussian component, \eqn{\sigma_2} is the estimated standard deviation of the second Gaussian component (Li, 2021).
+#' Without loss of generality, \eqn{\mu_2 \ge \mu_1}, thus \eqn{\delta \ge 0}, is assumed.}
+#' \item{Options}{A replication of input arguments and other information.}
+#'
 #'
 #'
 #' @author Seewoo Li \email{cu@@yonsei.ac.kr}
@@ -103,6 +120,8 @@
 #' @references
 #' Bock, R. D., & Aitkin, M. (1981). Marginal maximum likelihood estimation of item parameters: Application of an EM algorithm. \emph{Psychometrika, 46}(4), 443-459.
 #'
+#' Casabianca, J. M., & Lewis, C. (2015). IRT item parameter recovery with marginal maximum likelihood estimation using loglinear smoothing models. \emph{Journal of Educational and Behavioral Statistics, 40}(6), 547-578.
+#'
 #' Li, S. (2021). Using a two-component normal mixture distribution as a latent distribution in estimating parameters of item response models. \emph{Journal of Educational Evaluation, 34}(4), 759-789.
 #'
 #' Li, S. (2022). \emph{The effect of estimating latent distribution using kernel density estimation method on the accuracy and efficiency of parameter estimation of item response models} [Master's thesis, Yonsei University, Seoul]. Yonsei University Library.
@@ -149,12 +168,28 @@
 #'                   h=4 # an argument required only when "latent_dist = 'DC'"
 #'                   )
 #'
-IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
-                        latent_dist="Normal", max_iter=200, threshold=0.0001,
-                        bandwidth="SJ-ste", h=NULL){
+IRTest_Poly <- function(data, model="GPCM", range = c(-6,6), q = 121, initialitem=NULL,
+                        ability_method = 'EAP', latent_dist="Normal",
+                        max_iter=200, threshold=0.0001,bandwidth="SJ-ste",h=NULL){
+
+  categories <- apply(data, MARGIN = 2, FUN = extract_cat)
+
+  data <- reorder_mat(as.matrix(data))
+  if(is.null(initialitem)){
+    category <- apply(data, 2, max, na.rm = TRUE)
+    initialitem <- matrix(nrow = ncol(data), ncol = 7)
+    initialitem[,1] <- 1
+    initialitem[,2] <- 0
+    for(i in 3:7){
+      initialitem[category>(i-2),i] <- 0
+    }
+  }
+
   Options = list(initialitem=initialitem, data=data, range=range, q=q, model=model,
-                 latent_dist=latent_dist, max_iter=max_iter, threshold=threshold,
-                 bandwidth=bandwidth,h=h)
+                 ability_method=ability_method,latent_dist=latent_dist,
+                 max_iter=max_iter, threshold=threshold,bandwidth=bandwidth,h=h,
+                 categories=categories)
+
   I <- initialitem
   Xk <- seq(range[1],range[2],length=q)
   Ak <- dist2(Xk, 0.5, 0, 1)/sum(dist2(Xk, 0.5, 0, 1))
@@ -164,7 +199,7 @@ IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
   d = 1
   sd_ratio = 1
   N = nrow(data)
-  bw <- NULL
+  density_par <- NULL
 
   # Normality assumption method
   if(latent_dist %in% c("Normal", "normal", "N")){
@@ -174,6 +209,14 @@ IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
       E <- Estep_Poly(item=initialitem, data=data, q=q, prob=0.5, d=0, sd_ratio=1, range=range)
       M1 <- Mstep_Poly(E, item=initialitem, model=model)
       initialitem <- M1[[1]]
+
+      if(model == "PCM"){
+        ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range)
+        initialitem[,1] <- initialitem[,1]*ld_est$s
+        initialitem[,-1] <- initialitem[,-1]/ld_est$s
+        M1[[2]][,-1] <- M1[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(abs(I-initialitem), na.rm = TRUE)
       I <- initialitem
       message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
@@ -195,6 +238,12 @@ IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
       Xk <- ld_est$Xk
       Ak <- ld_est$posterior_density
 
+      if(model == "PCM"){
+        initialitem[,1] <- initialitem[,1]*ld_est$s
+        initialitem[,-1] <- initialitem[,-1]/ld_est$s
+        M1[[2]][,-1] <- M1[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(abs(I-initialitem), na.rm = TRUE)
       I <- initialitem
 
@@ -212,13 +261,20 @@ IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
       M1 <- Mstep_Poly(E, item=initialitem, model=model)
       initialitem <- M1[[1]]
       M2 <- M2step(E)
-      prob = M2[1];d = M2[3];sd_ratio = M2[4]
+      prob = M2$prob; d = M2$d; sd_ratio = M2$sd_ratio
       diff <- max(abs(I-initialitem), na.rm = TRUE)
-      I <- initialitem
+
+      if(model == "PCM"){
+        initialitem[,1] <- initialitem[,1]*M2$s
+        initialitem[,-1] <- initialitem[,-1]/M2$s
+        M1[[2]][,-1] <- M1[[2]][,-1]/M2$s
+      }
+
       message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
       flush.console()
     }
     Ak <- E$Ak
+    density_par <- list(prob=prob, d=d, sd_ratio=sd_ratio)
   }
 
   # Kernel density estimation method
@@ -235,18 +291,24 @@ IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
       Xk <- ld_est$Xk
       Ak <- ld_est$posterior_density
 
+      if(model == "PCM"){
+        initialitem[,1] <- initialitem[,1]*ld_est$s
+        initialitem[,-1] <- initialitem[,-1]/ld_est$s
+        M1[[2]][,-1] <- M1[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(abs(I-initialitem), na.rm = TRUE)
       I <- initialitem
 
       message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
       flush.console()
     }
-    bw <- ld_est$bw
+    density_par <- ld_est$par
   }
 
   # Davidian curve method
   if(latent_dist %in% c("DC", "Davidian")){
-    phipar <- nlminb(start = rep(1,h),
+    density_par <- nlminb(start = rep(1,h),
                      objective = optim_phi,
                      gradient = optim_phi_grad,
                      hp=h,
@@ -260,22 +322,68 @@ IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
       M1 <- Mstep_Poly(E, item=initialitem, model = model)
       initialitem <- M1[[1]]
 
-      ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range, phipar=phipar)
+      ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range, par=density_par)
       Xk <- ld_est$Xk
       Ak <- ld_est$posterior_density
 
+      if(model == "PCM"){
+        initialitem[,1] <- initialitem[,1]*ld_est$s
+        initialitem[,-1] <- initialitem[,-1]/ld_est$s
+        M1[[2]][,-1] <- M1[[2]][,-1]/ld_est$s
+      }
+
       diff <- max(abs(I-initialitem), na.rm = TRUE)
       I <- initialitem
 
       message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
       flush.console()
     }
+    density_par <- ld_est$par
   }
+  # Log-linear smoothing
+  if(latent_dist=="LLS"){
+    density_par <- rep(0, h)
+    while(iter < max_iter & diff > threshold){
+      iter <- iter +1
+
+      E <- Estep_Poly(item=initialitem, data=data, q=q, prob=0.5, d=0, sd_ratio=1,
+                      range=range, Xk=Xk, Ak=Ak)
+      M1 <- Mstep_Poly(E, item=initialitem, model=model)
+      initialitem <- M1[[1]]
+
+      ld_est <- latent_dist_est(method = latent_dist, Xk = E$Xk, posterior = E$fk, range=range, par=density_par, N=N)
+      Xk <- ld_est$Xk
+      Ak <- ld_est$posterior_density
+
+      if(model == "PCM"){
+        initialitem[,1] <- initialitem[,1]*ld_est$s
+        initialitem[,-1] <- initialitem[,-1]/ld_est$s
+        M1[[2]][,-1] <- M1[[2]][,-1]/ld_est$s
+      }
+
+      diff <- max(abs(I-initialitem), na.rm = TRUE)
+      I <- initialitem
+
+      message("\r","\r","Method = ",latent_dist,", EM cycle = ",iter,", Max-Change = ",diff,sep="",appendLF=FALSE)
+      flush.console()
+    }
+    density_par <- ld_est$par
+  }
+
+  # ability parameter estimation
+  if(ability_method == 'EAP'){
+    theta <- as.numeric(E$Pk%*%E$Xk)
+    theta_se <- NULL
+  } else if(ability_method == 'MLE'){
+    mle_result <- MLE_theta(item = initialitem, data = data, type = "poly")
+    theta <- mle_result[[1]]
+    theta_se <- mle_result[[2]]
+  }
+
   colnames(initialitem) <- c("a", "b_1", "b_2", "b_3", "b_4", "b_5", "b_6")
   colnames(M1[[2]]) <- c("a", "b_1", "b_2", "b_3", "b_4", "b_5", "b_6")
 
   # preparation for outputs
-  EAP <- as.numeric(E$Pk%*%E$Xk)
   logL <- 0
   for(i in 1:q){
     logL <- logL+sum(logLikeli_Poly(initialitem, data, theta = Xk[i])*E$Pk[,i])
@@ -288,16 +396,14 @@ IRTest_Poly <- function(initialitem, data, range = c(-6,6), q = 121, model,
          se=M1[[2]],
          fk=E$fk,
          iter=iter,
-         prob=prob,
-         d=d,
-         sd_ratio=sd_ratio,
          quad=Xk,
          diff=diff,
          Ak=Ak,
          Pk=E$Pk,
-         theta = EAP,
+         theta = theta,
+         theta_se = theta_se,
          logL=-2*logL, # deviance
-         bw=bw,
+         density_par = density_par,
          Options = Options # specified argument values
          ),
     class = c("poly", "irtest", "list")
